/**
 * This is the bridge for remote iframes to communicate with local windows
 * Remote windows use $desktop to access this bridge
 */

(function(_) {
  var emptyFunction = function() {
  
  }
  window.DesktopBridge = function(_private) {
    _.extend(this, {
    /**
     * use objPath as a dot notation lookup for an object
     * if objPath is a number, it is used to look up an instance in instanceManager
     *
     * example:
     * get('document.location.href')
     * get(123)
     * get(123,'instanceProperty.foo')
     *
     */
    get: function(objPath, start) {
      if (_.isString(objPath)) {
        var names = objPath.split('.'),
        obj = start || window;
        _.each(names, function(name) {
          obj = obj[name];
        });
        return obj;
      }
      else if (_.isNumber(objPath)) {
        var obj = window.instanceManager.get(objPath);
        if (start) {
          return this.get(start, obj);
        }
        else {
          return obj;
        }
      }
    }
    ,
    /**
     * use objPath as a lookup to an object and function.
     * the function will be invoked, using the object as the context
     * any following arguments will be passed as arguments
     * and the result returned
     *
     * objPath can be a dot notation string or a number referring to an instance ID
     *
     * examples:
     *
     * invoke('document.addAttribute', 'a-name', 'a-value');
     * invoke(123, 'methodName', arg1, arg2);
     * invoke(456, 'property.foo.bar.method', arg1, arg2)
     */
    invoke: function(objPath) {
      var path = objPath,
          obj = window,
          splicePos = 1;
      if (_.isNumber(objPath)) {
        obj = window.instanceManager.get(objPath);
        path = arguments[1];
        splicePos = 2;
      }
      var names = path.split('.'),
          stop = names.length-1;;
      
      for (var i=0;i<stop;i++) {
        obj = obj[names[i]];
      }
      var fn = obj[names[stop]];
      return fn.apply(obj, Array.prototype.slice.call(arguments, splicePos));
    }
    ,
    /**
     * Same as invoke, but the target is the remote in another window
     * respects the window ready state
     */
    windowInvoke: function(windowId) {
      var args = Array.prototype.slice.call(arguments, 1),
          targetWin = this._getWindow(windowId);
      
      if (targetWin.isReady()) {
        return targetWin.$remote.invoke.apply(targetWin.$remote, args);
      }
      else {
        targetWin.onReady( function() {
          targetWin.$remote.invoke.apply(targetWin.$remote, args);
        });
      }
    }
    ,
    /**
     * Same as get, but the target is the remote in another window
     * respects the window ready state
     */
    windowGet: function(windowId) {
      var args = Array.prototype.slice.call(arguments, 1),
          targetWin = this._getWindow(windowId);
          
      if (targetWin.isReady()) {
        return targetWin.$remote.get.apply(targetWin.$remote, args);
      }
      else {
        throw "windowGet: window not yet ready";
      }
    }
    ,
    /**
     * add an event listener to the target object
     * targetObjPath is an object path, or it can be a properties object
     * eventProps are the list of properties of the event object that the caller needs access to
     * preventDefault=true will prevent the event default
     * preventDefault=function(){} will execute the function to determine whether to prevent default
     * 
     */
    addEventListener: function(targetObjPath, eventName, fn, eventProps, preventDefault) {
      var targetObj = this.get(targetObjPath);
      
      if (!targetObj) {
        console.log('could not find target object for listener: ' + targetObjPath);
        return;
      }
      if (eventName.indexOf('.') > 0 ) {
        eventName = this.get(eventName);
      }
      var fnWrapper = _.bind(function(evt) {
        try {
          if (preventDefault === true ||
             (typeof(preventDefault) == 'function' && preventDefault())) {
            evt.preventDefault();
          }
          fn(this._copyTransferrable(evt, eventProps));
        }
        catch(ignore) {
          // errors can happen when trying to execute functions during exit
        }
        
      }, this);
      
      var functionId = window.instanceManager.add(fnWrapper);
      
      targetObj.addEventListener(eventName, fnWrapper, false);
      
      return functionId;
    }
    ,
    /**
     * remove an event listener to the target object
     * targetObjPath is an object path, or it can be a properties object
     * TODO: Verify this works properly
     */
    removeEventListener: function(targetObjPath, eventName, functionId) {
      var targetObj = this.get(targetObjPath);
      
      var fn = window.instanceManager.get(functionId);
      if (!fn) {
        return; // here we assume the function has already been removed
      }
      if (!targetObj) {
        console.log('could not find target object for listener: ' + targetObjPath);
        return;
      }
      if (eventName.indexOf('.') > 0 ) {
        eventName = this.get(eventName);
      }
      targetObj.removeEventListener(eventName, fn, false);
      window.instanceManager.remove(functionId);
    }
    ,
    /**
     * Add an error listener, a special version of addEventListener
     */
    addExceptionListener: function(fn) {
      try {
        var fnWrapper = function(evt) {
          var props = {
            message: evt.exceptionValue+''
          };
          
          var msg = evt.exceptionValue;
          if (evt.stackTrace && evt.stackTrace.length) {
            var trace = evt.stackTrace[0];
            props.sourceURL = trace.sourceURL;
            props.line = trace.line;
            props.functionName = trace.functionName;
          }
          fn(props);
        };
        
        var functionId = window.instanceManager.add(fnWrapper);
        window.htmlLoader.addEventListener("uncaughtScriptException", fnWrapper, false);
        
        return functionId;
      }
      catch (ex) {
        try {
          air.trace("error processing exception" + ex);
        }
        catch (will_loop_forever_if_not_caught) {
          
        }
      }
    }
    ,
    /**
     * copy strings, numbers, and booleans
     * The propertyList is an optional list of keys
     * This list should be provided in the cases where the the object cannot access the keys 
     * via the "for in" loop
     */
    _copyTransferrable: function(obj, propertyList) {
      var copy = {};
      if (propertyList) {
        _.each(propertyList, function(key) {
          var value = obj[key];
          if (_.isString(value) || _.isNumber(value) || _.isBoolean(value)) {
            copy[key] = value;
          }
        });
      }
      else {
        _.each(obj, function(value, key) {
          if (_.isString(value) || _.isNumber(value) || _.isBoolean(value)) {
            copy[key] = value;
          }
        });
      }
      return copy;
    }
    ,
    /**
     * removes an instance to clear up memory
     */
    clearInstance: function(id) {
      window.instanceManager.remove(id);
    }
    ,
    /**
     * parse the application descriptor and return the value for the given key
     */
    parseApplicationDescriptor: function (key) {
      var domParser = new DOMParser(),
      appDescriptor =  air.NativeApplication.nativeApplication.applicationDescriptor;

      return domParser.parseFromString(appDescriptor, 'text/xml').getElementsByTagName(key)[0].textContent;
    }
    ,
    getSharedObject: (function () {
      var sharedObject;
      return function() {
        return sharedObject || (sharedObject = air.SharedObject.getLocal('userSettings'));
      };
    })()
    ,
    getSecureStoreItem: function(key) {
      var so = this.getSharedObject();
      var item = so.data[key];

      if (!item) {
        return;
      }

      var value = item.toString();
      value = value === 'false' ? false : value;
      value = value === 'true'  ? true  : value;
      return value;
    }
    ,
    putSecureStoreItem: function(key, value) {
      if ((key+'').indexOf('secureServiceRoot') >=0) {
        if (!_.detect(_private.permittedSecureStoreUrls, function(url) {
          return value == url;
        })) {
          console.log('rejecting secureServiceRoot: ' + value);
          return;
        }
      }
      var so = this.getSharedObject();
      so.data[key] = new String(value);
      so.flush();
    }
    ,
    removeSecureStoreItem: function (key) {
      var so = this.getSharedObject();
      delete so.data[key];
    }
    ,
    /**
     * set the window title
     */
    setTitle: function(title) {
      window.title = title;
    }
    ,
    setWindowCoords: function (coords, windowId) {
      if (_.isNumber(coords.height)
          && _.isNumber(coords.width)
          && _.isNumber(coords.x)
          && _.isNumber(coords.y)) {
        var rect = new air.Rectangle(coords.x, coords.y, coords.width, coords.height);
        this._getWindow(windowId).nativeWindow.bounds = rect;
      }
    }
    ,
    getWindowCoords: function (windowId) {
      var domWindow = this._getWindow(windowId),
          bounds = domWindow.nativeWindow.bounds;
      return {
        x: bounds.x,
        y: bounds.y,
        width: bounds.width,
        height: bounds.height
      };
    }
    ,
    _getWindow: function(windowId) {
      var w = window.opener || window;
      return windowId ? w.instanceManager.get(windowId).window : w;
    }
    ,
    /**
     * sets the visibility of the target window
     * returns whether the value has changed
     */
    setWindowVisibility: function(visible, windowId) {
      var targetWin = this._getWindow(windowId);
      targetWin.nativeWindow.visible = visible;
    }
    ,
    /**
     * create a window using the given properties
     */
    createWindow: function (options, windowOptions) {
      var windowConfig  = new air.NativeWindowInitOptions(),
          coords = options.coords,
          rect = new air.Rectangle(coords.x, coords.y, coords.width, coords.height);
          
      _.extend(windowConfig, windowOptions);

      var html = air.HTMLLoader.createRootWindow(options.visible, windowConfig, true, rect);
      var windowId = window.instanceManager.add(html);

      console.log('window open: ' + options.url);
      
      html.load(new air.URLRequest(options.url));
      var onCompleteHandler = function(evt) {
        html.removeEventListener(air.Event.COMPLETE, onCompleteHandler, false);
        html.window.windowId = windowId;
      };
      html.addEventListener(air.Event.COMPLETE, onCompleteHandler , false);

      html.stage.nativeWindow.alwaysInFront = options.alwaysInFront;
      html.window.opener = window;

      return windowId;

    }
    ,
    setStartOnBoot: function (value) {
      try {
        air.NativeApplication.nativeApplication.startAtLogin = value;
      }
      catch (e) {
        //console.log('Tried to setStartOnBoot but failed. You may be running on adl');
      }
    }
    ,
    /**
     * A a menu item, the properties will specify where
     * 
     * Example props: 
     * {
     *   label: 'Logout',
     *   menuIndex: 0,
     *   index: 2,
     *   key: 'L',
     *   name: 'Logout'
     * }
     */
    addMenuItem: window.air && air.NativeApplication.supportsMenu ? 
      function(props, onSelect) {
        var menu = air.NativeApplication.nativeApplication.menu.items[props.menuIndex];
        this._addMenuItemInto(props, menu, onSelect)
      } 
      : emptyFunction
    ,
    /**
     * Add a menu divider at given position
     */
    addMenuDivider: window.air && air.NativeApplication.supportsMenu ? 
      function(menuIndex, itemIndex) {
        var menu = air.NativeApplication.nativeApplication.menu.items[menuIndex];
        menu.submenu.addItemAt(new air.NativeMenuItem("", true), itemIndex);
      } 
      : emptyFunction
    ,
    /**
     * Create and add a menu item into the given menu
     */
    _addMenuItemInto: window.air && air.NativeApplication.supportsMenu ? 
      function(props, menu, onSelect) {
        var menuItem = new air.NativeMenuItem(props.label);
        menuItem.keyEquivalent = props.key;
        menuItem.name = props.name || props.label;
        menuItem.addEventListener(air.Event.SELECT, function() {
          onSelect(props);
        }, false);
        if (menu.submenu && menu.submenu.addItemAt) {
          menu.submenu.addItemAt(menuItem, props.index);
        }
        else if (menu.addItemAt){
          menu.addItemAt(menuItem, props.index);
        }
      } 
      : emptyFunction
    ,
    /**
     * Create an entire menu, the config will specify where
     * 
     * Example config:
     * 
     * {
     *   index: 3,
     *   label: 'Font',
     *   items: [ {
     *       label: 'Larger',
     *       index: 0,
     *       key: '+'
     *     },
     *     {
     *       label: 'Smaller',
     *       index: 1,
     *       key: '-'
     *     },
     *     {
     *       label: 'Reset',
     *       index: 2,
     *       key: '0'
     *     }
     *   ]
     * }
     * 
     */
    addMenu: window.air && air.NativeApplication.supportsMenu ? 
      function(menuConfig, menuSelectionHandler) {
        var sysMenu = air.NativeApplication.nativeApplication.menu,
            menu = new air.NativeMenu();
        
        _.each(menuConfig.items, function(props, index){
          this._addMenuItemInto(props, menu, menuSelectionHandler);
        }, this);
        
        sysMenu.addSubmenuAt(menu, menuConfig.index, menuConfig.label);
      } 
      : emptyFunction
    ,
    /**
     * remove a menu item by name
     */
    removeMenuItemByName: window.air && air.NativeApplication.supportsMenu ? 
      function(menuIndex, name) {
        var menu = air.NativeApplication.nativeApplication.menu.items[menuIndex];
        menu.submenu.removeItem(menu.submenu.getItemByName(name));
      } 
      : emptyFunction
    ,
    /**
     * remote a menu item by index
     */
    removeMenuItem: window.air && air.NativeApplication.supportsMenu ? 
      function(menuIndex, itemIndex) {
        var menu = air.NativeApplication.nativeApplication.menu.items[menuIndex];
        menu.submenu.removeItemAt(itemIndex);
      } 
      : emptyFunction
    , 
    /**
     * remove a menu it the given index
     */
    removeMenu: window.air && air.NativeApplication.supportsMenu ? 
      function(menuIndex) {
        var sysMenu = air.NativeApplication.nativeApplication.menu;
        try {
          sysMenu.removeItemAt(menuIndex);
        } catch (e) {
          console.log(e)
        }
      } 
      : emptyFunction
    ,
    /**
     * read and load log file properties
     */
    getLogFileProperties: function() {
      var props = {};
      try {
        var file = air.File.applicationStorageDirectory.resolvePath('logs/log.txt');
        props.path = file.nativePath;
        props.url = file.url;
        props.userAgent = air.URLRequestDefaults.userAgent;
        
        var fileStream = new air.FileStream();
        fileStream.open(file, air.FileMode.READ);  // For Reading from file
        if ( file.exists ) { 
          props.data = fileStream.readMultiByte( fileStream.bytesAvailable, air.File.systemCharset );
        }
        fileStream.close();

      } catch (e) {
        console.error(e);
      }
      return props;
    }
    ,
    /**
     * open the url in the user's browser
     */
    openUrl: function(url) {
      try {
        console.log('opening in browser: ' + url);
        air.navigateToURL(new air.URLRequest(url));
      } catch (e) {
        console.error(e);
      }
    }
    ,
    /**
     * Create update handler
     */
    initAppUpdater: function() {
      window.appUpdater = new air.ApplicationUpdaterUI();
    }
    ,
    /**
     * Add param to appUpdater
     */
    setAppUpdaterParam: function(name, value) {
      window.appUpdater[name] = value;
    }
    ,
    /**
     * 
     */
    createContextMenu: function() {
      return window.instanceManager.add(new air.NativeMenu());
    }
    ,
    /**
     * add a menu item with a given name into the menu instance
     */
    addContextMenuItem: function(menuInstanceId, name) {
      var menu = window.instanceManager.get(menuInstanceId),
          item = new air.NativeMenuItem(name),
          itemId = window.instanceManager.add(item);
      
      menu.addItem(item); 
      return itemId;
    }
    ,
    /**
     * display a context menu at the given coordinates
     */
    displayContextMenu: function(menuInstanceId, x, y) {
      var contextMenu = window.instanceManager.get(menuInstanceId);
      contextMenu.display(window.nativeWindow.stage, x, y);
    }
    ,
    /**
     * explicitly log an event
     */
    log: function(info) {
      console.log(info);
    }
    
  });
};
})(_);
