/**
 * Local Bootstrap for Desktop
 * This bootstrap is used for all windows that have external iframes
 * Sets up bridges to remote iframes
 */


/**
 * Set up console.log
 */
if (typeof(console) == 'undefined') {
  if (typeof(air) != 'undefined' && air.Introspector) {
    console = {
      log: function(x) {
        air.Introspector.Console.log(x);
      }
    }
  }
  // if this is not the main window, send logs to the main window
  else if (opener && opener != window && opener.console) {
    console = {
      log: function(x) {
        opener.console.log(x);
      }
    }
  }
  // otherwise, just create an empty function so it doesn't error out
  else {
    console = {
      log: function() { }
    }
   }
}


(function(_, JSON) {

  /**
   * These private properties will not be directly accessible from remote windows
   */
  var _private = {
    localConfigurationFiles: [
      'config/defaults.json',
      'config/overrides.json'
    ]
    ,
    permittedSecureStoreUrls: [
      'https://www.yammer.com',
      'https://www.staging.yammer.com',
      ''
    ]
    ,
    staticAssetsUrl: '' // set via site_version
    ,
    baseUrl: '' // set via site_version
    ,
    remoteFrameId: 'remote-frame'
    ,
    hostenv: 'air'
    ,
    ready: false
    ,
    readyQueue: []
    ,
   /**
    * load local configuration from files
    */
   loadConfigs: function (){
      var config = {};
      _.each(arguments, function(path) {
        try {
          var json = this.readLocalFile(path),
          obj = JSON.parse(json);
        }
        catch(ignore) {
          // file may not exist
        }
        _.extend(config, obj);
      }, this);
      return config;
    }
    ,
    /**
     * read a local file
     */
    readLocalFile: function (path) {
      var file = air.File.applicationDirectory.resolvePath(path),
          stream = new air.FileStream(),
          bytes = new air.ByteArray();
          
      stream.open(file, air.FileMode.READ);
      stream.readBytes(bytes, 0, stream.bytesAvailable);
      stream.close();
      return bytes.toString();
    }
    ,
    /**
     * load the local preferences into this.configuration
     */
    loadLocalPrefs: function() {
      this.configuration = this.loadConfigs.apply(this, this.localConfigurationFiles);
    }
    ,
    /**
     * the iframe's id sould be remote-frame
     */
    getRemoteWindow: function() {
      return document.getElementById(this.remoteFrameId).contentWindow;
    }
    ,
    /**
     * should be called by the iframe ondominitialize event
     */
    connectRemote: function() {
      var remoteWin = this.getRemoteWindow();
      remoteWin.parentSandboxBridge = yam.desktop.local.bridge;
      if (remoteWin.childSandboxBridge) {
        remoteWin.childSandboxBridge.setParentBridgeReady();
      }
    }
    ,
    /**
     * Make an AJAX request locally
     */
    sendAjax: function(url, async, fn, onFailure) {
      try {
        var request = new XMLHttpRequest();
        if (async) {
          request.onreadystatechange = function(result) {
            if(result.readyState==4 && result.status==200) {
              fn(result);
            }
            else {
              onFailure(result);
            }
          }
          request.open('GET', url, async);
        }
        else {
          request.open('GET', url, false);
          request.send(null);
          if (request.status == 200){
            fn(request);
          }
          else {
            onFailure(request);
          }
        }
      }
      catch(e) {
        onFailure();
      }
    }
    ,
    /**
     * Make a ajax request
     */
    ajax: function(url, async, onSuccess, onFailure) {
      var self = this;
      
      this.sendAjax(url, async, onSuccess, function() {
          // WARNING: assuming dev mode here
          console.log('WARNING inserting image to handle bad certificate');
          try {
            var img = new Image();
            img.onerror = function() {
              self.sendAjax(url, async, onSuccess, function(result) {
                console.log('Failed XHR request to: ' + url);
                onFailure(result);
              })
            }
            img.src = url;
          }
          catch(e) {
            console.log('Failed XHR request to: ' + url);
            onFailure();
          }

      });

    }
    ,
    /**
     * load the remote page given a URI
     */
    loadPage: function(uri) {
      var f = document.createElement('iframe');
      f.id = this.remoteFrameId;
      
      this.connectFrame(f);
     
      var remoteRoot = window.getBaseUrl() + '/' + window.getSiteVersion();
      
      f.sandboxRoot = remoteRoot + '/local/';
      f.documentRoot="app:/sandbox/";
      f.setAttribute('allowCrossDomainXHR', 'allowCrossDomainXHR');

      var url = remoteRoot + '/' + uri;
      console.log('loading remote frame: ' + url);
      
      f.src = url;
      this.remotePageUrl = url;
      document.body.appendChild(f);
    }
    ,
    /**
     * reset the frame source to the stored remotePageUrl
     */
    reloadPage: function() {
      this.ready = false;
      window.instanceManager.clear();
      console.log('reloading remote frame: ' + this.remotePageUrl);
      document.getElementById(this.remoteFrameId).src = this.remotePageUrl;
    }
    ,
    /**
     * add a listener to connect the frame when the frame is ready
     */
    connectFrame: function(frame) {
      frame.ondominitialize = _.bind(function() {
        this.connectRemote();
      }, this);
    }
    ,
    parseHostEnvOptions: function() {
      var InvokeEvent = window.runtime.flash.events.InvokeEvent
        , nativeApplication = window.runtime.flash.desktop.NativeApplication.nativeApplication;

      nativeApplication.addEventListener(InvokeEvent.INVOKE, function (evt) {
        if (evt.arguments[0]) {
          var args = evt.arguments[0].split(' ');
          for (var i = 0, ii = args.length; i < ii; ++i) {
            if (args[i] == '--with-adl') {
              window.hostenvOptions.withAdl = true;
            }
          }
        }
      });
    }
    ,
    setUserAgent: function() {
      var appName = yam.desktop.local.bridge.parseApplicationDescriptor('version')
      , version = yam.desktop.local.bridge.parseApplicationDescriptor('filename').replace(/ /g, '_');

      var append = ' ' + appName + '/' + version;
      try {
        air.URLRequestDefaults.userAgent += append;
      } catch (e) {
        // I'm pretty sure this used to work at one point
        window.htmlLoader.userAgent += append;
      }
    }
    ,
    initDockIcon: function (){
      var ApplicationIcon = window.runtime.com.yammer.ui.ApplicationIcon;
      var dockIcon = window.dockIcon = new ApplicationIcon(
          'resources/images/appicons/air_128x128.png'
          , 'resources/images/appicons/air_16x16.png'
          , 'resources/images/appicons/air_16x16_red.png'
        );
      dockIcon.addEventListener(ApplicationIcon.LOADED, function (evt){
      dockIcon.setTrayMenu();

        // On windows and linux, this will activate the application if the system tray icon is clicked
        if (air.NativeApplication.supportsSystemTrayIcon){
          air.NativeApplication.nativeApplication.icon.addEventListener(air.MouseEvent.CLICK, function (evt){
            window.nativeWindow.visible = true;
            window.nativeWindow.activate();
          });
        }
      });
      dockIcon.create();
    }
    ,
    /**
     * This is the version used to local all remote files
     */
    siteVersion: ''
    ,
    /**
     * This is the date of the most recent asset update
     */
    buildDate: ''
    ,
    /**
     * Make an ajax call to get information about the current site version
     * Use that information to set a valid siteVersion
     */
    setSiteVersion: function(onSuccess, onFailure) {
 
      if (opener && opener != window) {
        this.siteVersion = opener.getSiteVersion();
      }
      else {
        
        var url = window.getConfigurationValue('siteVersionUrl');
        console.log('Loading site version info: ' + url);
        this.ajax(url, false, _.bind(function(result) {
            if (result.responseText.indexOf('<!DOCTYPE html>') < 0) {
              try {
                var json = JSON.parse(result.responseText);
                // load user pref
                var userSiteVersion = yam.desktop.local.bridge.getSecureStoreItem('siteVersion');
                if (userSiteVersion && _.detect(json.supported, function(v) {
                  return v==userSiteVersion;
                })) {
                  this.siteVersion = userSiteVersion;
                }
                else {
                  this.siteVersion = json.current;
                  this.buildDate = json.buildDate;
                }
                this.baseUrl = json.baseUrl;
                this.staticAssetsUrl = json.staticAssetsUrl;
                onSuccess();
                return;
              }
              catch(ex) {
                // let it fail below
              }
            }
            onFailure(result, 'maintenance');
            
        }, this), onFailure);

      }

    }
    ,
    init: function() {
      this.loadLocalPrefs();
      this.parseHostEnvOptions();
      this.setUserAgent();
    }
  };


  if (window.browserLocalOverrides) {
    _.extend(_private, window.browserLocalOverrides);
    delete window.browserLocalOverrides;
  }
  
  yam.desktop.local.bridge = new DesktopBridge(_private);
  
  if (window.BrowserBridgeOverrides) {
    _.extend(yam.desktop.local.bridge, new window.BrowserBridgeOverrides(_private));
    delete window.browserBridgeOverrides;
  }
  
  /**
   * these public properties will be accessible by remote windows through the bridge
   */
  _.extend(window, {
    /**
     * called by the remote iframe (through the bridge)
     */
    onChildBridgeReady: function() {
      window.$remote = _private.getRemoteWindow().childSandboxBridge;
    }
    ,
    /**
     * set that the remote frame is ready for interaction
     * also sets the title
     * calls any queue of functions that are waiting
     */
    setReady: function(title) {
      _private.ready = true;
      _.each(_private.readyQueue, function(fn) {
        fn();
      });
    }
    ,
    /**
     * is the remote frame ready
     */
    isReady: function() {
      return _private.ready;
    }
    ,
    /**
     * add a listener to call when the remote frame is ready
     */
    onReady: function(fn) {
     if (!this.isReady()) {
       _private.readyQueue.push(fn);
     }
     else {
        return fn();
     } 
    }
    ,
    /**
     * get a configuration value
     */
    getConfigurationValue: function(key) {
      return _private.configuration[key];
    }
    ,
    /**
     * copy the configuration options
     * this is to prevent direct access to the configuration object
     */
    copyConfiguration: function() {
      var str = JSON.stringify(_private.configuration);
      return JSON.parse(str);
    }
    ,
    hostenv: _private.hostenv
    ,
    /**
     * get the site version used to load remote resources
     */
    getSiteVersion: function() {
      if (opener && opener != window) {
        return opener.getSiteVersion();
      }
      else {
        return _private.siteVersion;
      }
    }
    ,
    /**
     * get the date of the last asset deploy 
     */
    getBuildDate: function() {
      if (opener && opener != window) {
        return opener.getBuildDate();
      }
      else {
        return _private.buildDate;
      }
    }
    ,
    getStaticAssetsUrl: function() {
      if (opener && opener != window) {
        return opener.getStaticAssetsUrl();
      }
      else {
        return _private.staticAssetsUrl;
      }
    }
    ,
    getBaseUrl: function() {
      if (opener && opener != window) {
        return opener.getBaseUrl();
      }
      else {
        return _private.baseUrl;
      }
    }
    ,
    /**
     * Set the frame rate (not sure that this works as documented)
     */
    setFramerate: function(rate) {
      if (window.nativeWindow && window.nativeWindow.stage) {
        window.nativeWindow.stage.frameRate = rate;
      }
    }
    ,
    hostenvOptions: {}
    ,
    /**
     * an instance manager for remotely tracking instances in this window
     * NOTE: will memory leak if instances are not removed
     */
    instanceManager: (function() {
      var instanceManagerCounter=  1;
      var instances = {};
      return {
        clear: function() {
          instanceManagerCounter=  1;
          instances = {};
        },
        add: function(obj) {
          var id = instanceManagerCounter++;
          instances[id] = obj;
          return id;
        },
        get: function(id) {
          return instances[id];
        },
        nextId: function() {
          return instanceManagerCounter++;
        },
        put: function(id, obj) {
          instances[id] = obj;
        },
        remove: function(id) {
          delete instances[id];
        }
      };
    })()
    ,
    /**
     * When connection is lost call this function to display lost connection message
     */
    noConnection: function() {
      window.nativeWindow.visible = true;
      window.nativeWindow.activate();
      document.location = 'NoConnection.html' + document.location.search;
    }
    ,
    /**
     * When the site is known to be down for maintenance, call this function to display the appropriate message
     */
    siteMaintenance: function() {
      window.nativeWindow.visible = true;
      window.nativeWindow.activate();
      document.location = 'NoConnection.html' + document.location.search + '#reason=maintenance';
    }
    ,
    /**
     * reload the iframe and reinitialize it
     */
    reloadPage: function() {
      _private.reloadPage();
    }
  });
  
  
  _private.init();
  
  /**
   * load the remote frame passed through the URL
   */
  window.addEventListener('load', function() {
    window.setFramerate(60);
    if (window.location.search) {
      var list = window.location.search.substr(1).split('&');
      var params = {};
      for (var i=0;i<list.length;i++) {
        var pair = list[i].split('=');
        params[pair[0]] = decodeURIComponent(pair[1]);
      }
      if (params.main && params.main == 'true') {
        
        // on some platforms, the min size is applied to include the window frame
        // so an adjustment needs to be made
        var minSize = window.nativeWindow.minSize;
        if (minSize && document.body.clientWidth < minSize.x) {
          
          var dx = (minSize.x-document.body.clientWidth);
          var x = minSize.x+dx;
          console.log('Readjusting minimum width: ' + x);
          window.nativeWindow.minSize = new air.Point(x, minSize.y);
        }
        _private.initDockIcon();
         // This means we need to check the version number
         _private.setSiteVersion(function() {
          if (params.page) {
            _private.loadPage(params.page);
          }
        }, function(result, reason) {
          if (reason==='maintenance' || (result && result.status==503)) {
            window.siteMaintenance();
          }
          else {
            window.noConnection();
          }
        });
        return;
      }
      if (params.page) {
        _private.loadPage(params.page);
      }
      
    }
  }, false);
})(_ , JSON);
